'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = normalize;

var _blacklist = require('blacklist');

var _blacklist2 = _interopRequireDefault(_blacklist);

var _erschema = require('../erschema');

var _lodash = require('lodash.pick');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function normalize(input, entityName, schema, startingSchema) {
  var entities = {};
  var relationships = {};
  _normalizeRecursive(input, entityName, schema, entities, relationships, startingSchema);
  return { entities: entities, relationships: relationships };
}

var _normalizeRecursive = function _normalizeRecursive(input, entityName, schema, entities, relationshipData, startingSchema) {
  var entitySchema = startingSchema || schema[entityName];
  if (!entitySchema) {
    throw Error('schema ' + entityName + ' not defined');
  }
  var modifier = entitySchema.modifier,
      relationships = entitySchema.relationships,
      idFunc = entitySchema.idFunc,
      properties = entitySchema.properties;

  var usedRelationships = [];
  var inputId = idFunc(input);
  relationships.forEach(function (relationshipSchema) {
    var variableRelationshipName = relationshipSchema.variableRelationshipName,
        relationshipName = relationshipSchema.relationshipName,
        alias = relationshipSchema.alias;

    var variableIds = {};
    var relation = input[alias || relationshipName];

    if (relation) {
      usedRelationships.push(relationshipName);
      var relationshipIdFunc = schema[relationshipSchema.name].idFunc;

      if (relationshipSchema.type === _erschema.relationshipTypes.MANY) {
        var relationshipIds = [];
        relation.forEach(function (relatedEntity) {
          if (typeof relatedEntity === 'number') {
            relationshipIds.push(relatedEntity);
          } else {
            _normalizeRecursive(relatedEntity, relationshipSchema.name, schema, entities, relationshipData);
            if (variableRelationshipName && variableRelationshipName.getRelationshipName) {
              var variableName = variableRelationshipName.getRelationshipName(relatedEntity);
              if (!variableIds[variableName]) {
                variableIds[variableName] = [];
              }
              variableIds[variableName].push(relationshipIdFunc(relatedEntity));
            }
            relationshipIds.push(relationshipIdFunc(relatedEntity));
          }
        });
        if (variableRelationshipName && variableRelationshipName.getRelationshipName) {
          Object.keys(variableIds).forEach(function (variableRelationshipName) {
            _addToRelationships(relationshipData, entityName, variableRelationshipName, inputId, variableIds[variableRelationshipName]);
          });
        }
        _addToRelationships(relationshipData, entityName, relationshipName, inputId, relationshipIds);
      } else {
        var relationshipId = void 0;
        if (typeof relation === 'number') {
          relationshipId = relation;
        } else {
          relationshipId = relationshipIdFunc(relation);
          _normalizeRecursive(relation, relationshipSchema.name, schema, entities, relationshipData);
        }
        _addToRelationships(relationshipData, entityName, relationshipName, inputId, relationshipId);
      }
    }
  });
  _addToEntities(entities, entityName, usedRelationships, input, modifier, properties, inputId);
};

var _addToRelationships = function _addToRelationships(relationships, entityName, relationshipName, entityId, values) {
  if (!relationships[entityName]) {
    relationships[entityName] = {};
  }
  if (!relationships[entityName][relationshipName]) {
    relationships[entityName][relationshipName] = {};
  }
  relationships[entityName][relationshipName][entityId] = values;
};

var _addToEntities = function _addToEntities(entities, entityName, usedRelationships, entity, modifier, properties, id) {
  if (!entities[entityName]) {
    entities[entityName] = {};
  }

  var nextEntity = (0, _lodash2.default)(entity, properties);
  // if (usedRelationships.length) {
  //   nextEntity = blacklist.apply(null, [entity,...usedRelationships])
  // }
  var nextEntityWithId = _extends({}, nextEntity, { id: id });
  entities[entityName][id] = modifier ? modifier(nextEntityWithId) : nextEntityWithId;
};